@@ -10,6 +10,7 @@ from typing import (
     List,
     Literal,
     Optional,
+    Set,
 )
 
 from motion.execute import Executor
@@ -46,6 +47,7 @@ class ComponentInstance:
         disable_update_task: bool = False,
         cache_ttl: int = DEFAULT_KEY_TTL,
         redis_socket_timeout: int = 60,
+        flush_on_exit: bool = False,
     ):
         """Creates a new instance of a Motion component.
 
@@ -74,6 +76,13 @@ class ComponentInstance:
 
         self.running = False
         self.disable_update_task = disable_update_task
+        self.flush_on_exit = flush_on_exit
+
+        if self.disable_update_task and self.flush_on_exit:
+            raise ValueError("Cannot flush on exit if update task is disabled.")
+
+        self.flows_run: Set[str] = set()
+
         self._executor = Executor(
             self._instance_name,
             cache_ttl=self._cache_ttl,
@@ -146,6 +155,11 @@ class ComponentInstance:
         if not self.running:
             return
 
+        # Flush the update queue
+        if self.flush_on_exit:
+            for flow_key in self.flows_run:
+                self.flush_update(flow_key)
+
         is_open = is_logger_open(logger)
 
         if is_open:
@@ -255,8 +269,8 @@ class ComponentInstance:
         self._executor._loadState()
         return self._executor._state.get(key, default_value)
 
-    def flush_update(self, dataflow_key: str) -> None:
-        """Flushes the update queue corresponding to the dataflow
+    def flush_update(self, flow_key: str) -> None:
+        """Flushes the update queue corresponding to the flow
         key, if it exists, and updates the instance state.
         Warning: this is a blocking operation and could take
         a while if your update op takes a long time!
@@ -293,7 +307,7 @@ class ComponentInstance:
         ```
 
         Args:
-            dataflow_key (str): Key of the dataflow.
+            flow_key (str): Key of the flow.
 
         Raises:
             RuntimeError:
@@ -302,17 +316,17 @@ class ComponentInstance:
         if self.disable_update_task:
             raise RuntimeError("Cannot run a disable_update_task component instance.")
 
-        self._executor.flush_update(dataflow_key)
+        self._executor.flush_update(flow_key)
 
     def gen(
         self,
-        dataflow_key: str,
+        flow_key: str,
         props: Dict[str, Any] = {},
         ignore_cache: bool = False,
         force_refresh: bool = False,
         flush_update: bool = False,
     ) -> Generator[Any, None, None]:
-        """Runs the dataflow (serve and update ops) for the specified key and
+        """Runs the flow (serve and update ops) for the specified key and
         yields the results as they come in, as a generator. Use this if your
         serve op is a generator function. If your serve op just returns a
         value, use run instead. You should use agen as opposed to gen if your
@@ -342,9 +356,9 @@ class ComponentInstance:
         ```
 
         Args:
-            dataflow_key (str): Key of the dataflow to run.
+            flow_key (str): Key of the flow to run.
             props (Dict[str, Any]): Keyword arguments to pass into the
-                dataflow ops, in addition to the state.
+                flow ops, in addition to the state.
             ignore_cache (bool, optional):
                 If True, ignores the cache and runs the serve op. Does not
                 force refresh the state. Defaults to False.
@@ -359,7 +373,7 @@ class ComponentInstance:
                 state after the update op completes. Defaults to False.
 
         Raises:
-            ValueError: If more than one dataflow key-value pair is passed.
+            ValueError: If more than one flow key-value pair is passed.
                 If flush_update is called and the component instance update
                 processes are disabled.
 
@@ -367,7 +381,7 @@ class ComponentInstance:
             Awaitable[Any]: Awaitable Result of the serve call.
         """
         for elem in self._executor.run(
-            key=dataflow_key,
+            key=flow_key,
             props=props,
             ignore_cache=ignore_cache,
             force_refresh=force_refresh,
@@ -375,18 +389,20 @@ class ComponentInstance:
         ):  # type: ignore
             yield elem
 
+        self.flows_run.add(flow_key)
+
     def run(
         self,
         # *,
-        dataflow_key: str,
+        flow_key: str,
         props: Dict[str, Any] = {},
         ignore_cache: bool = False,
         force_refresh: bool = False,
         flush_update: bool = False,
     ) -> Any:
-        """Runs the dataflow (serve and update ops) for the keyword argument
+        """Runs the flow (serve and update ops) for the keyword argument
         passed in. If the key is not found to have any ops, an error
-        is raised. Only one dataflow key should be passed in.
+        is raised. Only one flow key should be passed in.
 
         Example Usage:
         ```python
@@ -419,15 +435,15 @@ class ComponentInstance:
 
         # 1. Waits for the update op to finish, then updates the state
         # 2. Returns 2, result state["value"] = 4
-        # 3. Force refreshes the state before running the dataflow, and
+        # 3. Force refreshes the state before running the flow, and
         #    reruns the serve op even though the result might be cached.
         ```
 
 
         Args:
-            dataflow_key (str): Key of the dataflow to run.
+            flow_key (str): Key of the flow to run.
             props (Dict[str, Any]): Keyword arguments to pass into the
-                dataflow ops, in addition to the state.
+                flow ops, in addition to the state.
             ignore_cache (bool, optional):
                 If True, ignores the cache and runs the serve op. Does not
                 force refresh the state. Defaults to False.
@@ -442,7 +458,7 @@ class ComponentInstance:
                 state after the update op completes. Defaults to False.
 
          Raises:
-            ValueError: If more than one dataflow key-value pair is passed.
+            ValueError: If more than one flow key-value pair is passed.
             RuntimeError:
                 If flush_update is called and the component instance update
                 processes are disabled.
@@ -455,7 +471,7 @@ class ComponentInstance:
 
         serve_result = []
         for elem in self.gen(
-            dataflow_key, props, ignore_cache, force_refresh, flush_update
+            flow_key, props, ignore_cache, force_refresh, flush_update
         ):
             serve_result.append(elem)
 
@@ -463,13 +479,13 @@ class ComponentInstance:
 
     async def agen(
         self,
-        dataflow_key: str,
+        flow_key: str,
         props: Dict[str, Any] = {},
         ignore_cache: bool = False,
         force_refresh: bool = False,
         flush_update: bool = False,
     ) -> AsyncGenerator[Any, None]:
-        """Async version of gen. Runs the dataflow (serve and update ops) for
+        """Async version of gen. Runs the flow (serve and update ops) for
         the specified key and yields the results as they come in,
         as a generator. Use this if your serve op is an async
         generator function. You should use agen as opposed to gen if your
@@ -498,9 +514,9 @@ class ComponentInstance:
         ```
 
         Args:
-            dataflow_key (str): Key of the dataflow to run.
+            flow_key (str): Key of the flow to run.
             props (Dict[str, Any]): Keyword arguments to pass into the
-                dataflow ops, in addition to the state.
+                flow ops, in addition to the state.
             ignore_cache (bool, optional):
                 If True, ignores the cache and runs the serve op. Does not
                 force refresh the state. Defaults to False.
@@ -515,7 +531,7 @@ class ComponentInstance:
                 state after the update op completes. Defaults to False.
 
         Raises:
-            ValueError: If more than one dataflow key-value pair is passed.
+            ValueError: If more than one flow key-value pair is passed.
                 If flush_update is called and the component instance update
                 processes are disabled.
 
@@ -523,7 +539,7 @@ class ComponentInstance:
             Awaitable[Any]: Awaitable Result of the serve call.
         """
         async for elem in self._executor.arun(
-            key=dataflow_key,
+            key=flow_key,
             props=props,
             ignore_cache=ignore_cache,
             force_refresh=force_refresh,
@@ -531,16 +547,18 @@ class ComponentInstance:
         ):  # type: ignore
             yield elem
 
+        self.flows_run.add(flow_key)
+
     async def arun(
         self,
         # *,
-        dataflow_key: str,
+        flow_key: str,
         props: Dict[str, Any] = {},
         ignore_cache: bool = False,
         force_refresh: bool = False,
         flush_update: bool = False,
     ) -> Awaitable[Any]:
-        """Async version of run. Runs the dataflow (serve and update ops) for
+        """Async version of run. Runs the flow (serve and update ops) for
         the specified key. You should use arun if either the serve or update op
         is an async function.
 
@@ -565,9 +583,9 @@ class ComponentInstance:
         ```
 
         Args:
-            dataflow_key (str): Key of the dataflow to run.
+            flow_key (str): Key of the flow to run.
             props (Dict[str, Any]): Keyword arguments to pass into the
-                dataflow ops, in addition to the state.
+                flow ops, in addition to the state.
             ignore_cache (bool, optional):
                 If True, ignores the cache and runs the serve op. Does not
                 force refresh the state. Defaults to False.
@@ -582,7 +600,7 @@ class ComponentInstance:
                 state after the update op completes. Defaults to False.
 
         Raises:
-            ValueError: If more than one dataflow key-value pair is passed.
+            ValueError: If more than one flow key-value pair is passed.
                 If flush_update is called and the component instance update
                 processes are disabled.
 
@@ -594,7 +612,7 @@ class ComponentInstance:
         results = []
 
         async for elem in self.agen(
-            dataflow_key, props, ignore_cache, force_refresh, flush_update
+            flow_key, props, ignore_cache, force_refresh, flush_update
         ):
             results.append(elem)
 
